/*
 * linux/include/asm-arm/arch-sa1100/time.h
 *
 * Copyright (C) 1998 Deborah Wallach.
 * Twiddles  (C) 1999 	Hugo Fiennes <hugo@empeg.com>
 * 			Nicolas Pitre <nico@cam.org>
 */

#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>

static volatile unsigned int last_os_timer = 0;

/* IRQs are disabled before entering here from do_gettimeofday() */
static inline long gettimeoffset (void)
{
	/* Vastly simplified workings here: instead of trying to work out if there
	   has been an unserviced interrupt during this time, our reset_timer()
	   routine just keeps track of the *last* timer match value (ie, the one
	   where we last updated the OS timer). Even if we've missed an interrupt,
	   this should be fine, as the offset we return here will be greater than
	   one tick interval (ie LATCH).
	   hugo@empeg.com 990703
	*/
	unsigned long offset, usec;

	/* Get ticks since last timer service */
	offset=OSCR-last_os_timer;
	
	usec = (unsigned long)(offset*tick)/LATCH;

	return usec;
}

static inline void reset_timer (void)
{
	int next_os_timer_match;
	unsigned long flags;

	/* Disable IRQs during this timer update: this is done because previously
	   problems could occur with an IRQ from another source interrupting
	   this update routine, causing the match value to have been passed by
	   the time the OSMR was written to.
	*/
	save_flags_cli(flags);
	OSSR = 1;  /* Clear match on timer 0 */
	next_os_timer_match = LATCH + (last_os_timer = OSCR);
	OSMR0 = next_os_timer_match;
	restore_flags(flags);
}

/*
 * timer_interrupt() needs to keep up the real-time clock,
 * as well as call the "do_timer()" routine every clocktick.
 */
static void timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	reset_timer();
	do_timer(regs);
}

static struct irqaction irqtimer = 
	{timer_interrupt, 0, 0, "timer", NULL, NULL};

static inline unsigned long setup_timer (void)
{
	OSSR = 0xf;		/* clear status on all timers */
	setup_arm_irq(IRQ_OST0, &irqtimer);
	OSCR = 0;		/* initialize free-running timer register */
	OIER |= OIER_E0;	/* enable match on timer 0 to cause interrupts */
	reset_timer();

	/*
	 * Default the date to 1 Jan 1970 0:0:0
	 * You will have to run a time daemon to set the
	 * clock correctly at bootup
	 */
	return mktime(1970, 1, 1, 0, 0, 0);
}

